@@ -46,6 +46,7 @@ from vsc.utils.missing import all
 
 import easybuild.tools.environment as env
 from easybuild.tools.build_log import print_msg  # import build_log must stay, to activate use of EasyBuildLog
+from easybuild.tools.config import build_option
 from easybuild.tools import run
 
 
@@ -175,12 +176,7 @@ def extract_file(fn, dest, cmd=None, extra_options=None, overwrite=False):
     if not os.path.isfile(fn):
         _log.error("Can't extract file %s: no such file" % fn)
 
-    if not os.path.isdir(dest):
-        # try to create it
-        try:
-            os.makedirs(dest)
-        except OSError, err:
-            _log.exception("Can't extract file %s: directory %s can't be created: %err " % (fn, dest, err))
+    mkdir(dest, parents=True)
 
     # use absolute pathnames from now on
     absDest = os.path.abspath(dest)
@@ -203,7 +199,7 @@ def extract_file(fn, dest, cmd=None, extra_options=None, overwrite=False):
     if extra_options:
         cmd = "%s %s" % (cmd, extra_options)
 
-    run_cmd(cmd, simple=True)
+    run.run_cmd(cmd, simple=True)
 
     return find_base_dir()
 
@@ -249,8 +245,7 @@ def download_file(filename, url, path):
 
     # make sure directory exists
     basedir = os.path.dirname(path)
-    if not os.path.exists(basedir):
-        os.makedirs(basedir)
+    mkdir(basedir, parents=True)
 
     downloaded = False
     attempt_cnt = 0
@@ -307,19 +302,15 @@ def find_easyconfigs(path, ignore_dirs=None):
     return files
 
 
-def search_file(paths, query, build_options=None, short=False):
+def search_file(paths, query, short=False, ignore_dirs=None, silent=False):
     """
     Search for a particular file (only prints)
     """
-    if build_options is None:
-        build_options = {}
-
-    ignore_dirs = build_options.get('ignore_dirs', ['.git', '.svn'])
+    if ignore_dirs is None:
+        ignore_dirs = ['.git', '.svn']
     if not isinstance(ignore_dirs, list):
         _log.error("search_file: ignore_dirs (%s) should be of type list, not %s" % (ignore_dirs, type(ignore_dirs)))
 
-    silent = build_options.get('silent', False)
-
     var_lines = []
     hit_lines = []
     var_index = 1
@@ -616,7 +607,7 @@ def apply_patch(patchFile, dest, fn=None, copy=False, level=None):
         _log.debug("Using specified patch level %d for patch %s" % (level, patchFile))
 
     patchCmd = "patch -b -p%d -i %s" % (p, apatch)
-    result = run_cmd(patchCmd, simple=True)
+    result = run.run_cmd(patchCmd, simple=True)
     if not result:
         _log.error("Patching with patch %s failed" % patchFile)
         return
@@ -745,32 +736,53 @@ def patch_perl_script_autoflush(path):
     write_file(path, newtxt)
 
 
-def mkdir(directory, parents=False):
+def mkdir(path, parents=False, set_gid=None, sticky=None):
     """
     Create a directory
     Directory is the path to create
 
-    When parents is True then no error if directory already exists
-    and make parent directories as needed (cfr. mkdir -p)
+    @param parents: create parent directories if needed (mkdir -p)
+    @param set_gid: set group ID bit, to make subdirectories and files inherit group
+    @param sticky: set the sticky bit on this directory (a.k.a. the restricted deletion flag),
+                   to avoid users can removing/renaming files in this directory
     """
-    if parents:
+    if set_gid is None:
+        set_gid = build_option('set_gid_bit')
+    if sticky is None:
+        sticky = build_option('sticky_bit')
+
+    # exit early if path already exists
+    if not os.path.exists(path):
+        tup = (path, parents, set_gid, sticky)
+        _log.info("Creating directory %s (parents: %s, set_gid: %s, sticky: %s)" % tup)
+        # set_gid and sticky bits are only set on new directories, so we need to determine the existing parent path
+        existing_parent_path = os.path.dirname(path)
         try:
-            os.makedirs(directory)
-            _log.debug("Succesfully created directory %s and needed parents" % directory)
-        except OSError, err:
-            if err.errno == errno.EEXIST:
-                _log.debug("Directory %s already exitst" % directory)
+            if parents:
+                # climb up until we hit an existing path or the empty string (for relative paths)
+                while existing_parent_path and not os.path.exists(existing_parent_path):
+                    existing_parent_path = os.path.dirname(existing_parent_path)
+                os.makedirs(path)
             else:
-                _log.error("Failed to create directory %s: %s" % (directory, err))
-    else:  # not parents
-        try:
-            os.mkdir(directory)
-            _log.debug("Succesfully created directory %s" % directory)
+                os.mkdir(path)
         except OSError, err:
-            if err.errno == errno.EEXIST:
-                _log.warning("Directory %s already exitst" % directory)
-            else:
-                _log.error("Failed to create directory %s: %s" % (directory, err))
+            _log.error("Failed to create directory %s: %s" % (path, err))
+
+        # set group ID and sticky bits, if desired
+        bits = 0
+        if set_gid:
+            bits |= stat.S_ISGID
+        if sticky:
+            bits |= stat.S_ISVTX
+        if bits:
+            try:
+                new_subdir = path[len(existing_parent_path):].lstrip(os.path.sep)
+                new_path = os.path.join(existing_parent_path, new_subdir.split(os.path.sep)[0])
+                adjust_permissions(new_path, bits, add=True, relative=True, recursive=True, onlydirs=True)
+            except OSError, err:
+                _log.error("Failed to set groud ID/sticky bit: %s" % err)
+    else:
+        _log.debug("Not creating existing path %s" % path)
 
 
 def rmtree2(path, n=3):
