@@ -29,11 +29,12 @@ Unit tests for talking to GitHub.
 """
 
 import os
-from unittest import TestCase, TestLoader, main
-
+from test.framework.utilities import EnhancedTestCase
+from unittest import TestLoader, main
 
 from easybuild.tools.github import Githubfs
 
+
 # the user who's repo to test
 GITHUB_USER = "hpcugent"
 # the repo of this user to use in this test
@@ -47,14 +48,14 @@ GITHUB_TOKEN = os.environ.get('EASYBUILD_GITHUB_TOKEN', None)
 # branch to test
 GITHUB_BRANCH = 'master'
 
-class GithubTest(TestCase):
+class GithubTest(EnhancedTestCase):
     """ small test for The github package
     This should not be to much, since there is an hourly limit of request
     for non authenticated users of 50"""
 
     def setUp(self):
         """setup"""
-        self.cwd = os.getcwd()
+        super(GithubTest, self).setUp()
         self.ghfs = Githubfs(GITHUB_USER, GITHUB_REPO, GITHUB_BRANCH, GITHUB_LOGIN, GITHUB_PASSWORD, GITHUB_TOKEN)
 
     def test_walk(self):
@@ -84,10 +85,6 @@ class GithubTest(TestCase):
         except (IOError, OSError):
             pass
 
-    def tearDown(self):
-        """cleanup"""
-        os.chdir(self.cwd)
-
 def suite():
     """ returns all the testcases in this module """
     return TestLoader().loadTestsFromTestCase(GithubTest)
