@@ -30,46 +30,29 @@ Unit tests for modules.py.
 @author: Stijn De Weirdt (Ghent University)
 """
 
-import copy
 import os
 import re
 import tempfile
 import shutil
+from test.framework.utilities import EnhancedTestCase
+from unittest import TestLoader, main
 
-import easybuild.tools.options as eboptions
-from easybuild.tools import config
 from easybuild.tools.build_log import EasyBuildError
-from easybuild.tools.environment import modify_env
 from easybuild.tools.modules import get_software_root, get_software_version, get_software_libdir, modules_tool
-from unittest import TestCase, TestLoader, main
 
 
 # number of modules included for testing purposes
 TEST_MODULES_COUNT = 34
 
 
-class ModulesTest(TestCase):
+class ModulesTest(EnhancedTestCase):
     """Test cases for modules."""
 
-    def assertErrorRegex(self, error, regex, call, *args):
-        """ convenience method to match regex with the error message """
-        try:
-            call(*args)
-        except error, err:
-            self.assertTrue(re.search(regex, err.msg))
-
     def setUp(self):
         """set up everything for a unit test."""
-        # keep track of original environment, so we can restore it
-        self.orig_environ = copy.deepcopy(os.environ)
+        super(ModulesTest, self).setUp()
 
-        # initialize configuration so config.get_modules_tool function works
-        eb_go = eboptions.parse_options()
-        config.init(eb_go.options, eb_go.get_options_by_section('config'))
-
-        self.cwd = os.getcwd()
         self.orig_modulepaths = os.environ.get('MODULEPATH', '').split(os.pathsep)
-
         self.testmods = None
 
         # purge with original $MODULEPATH before running each test
@@ -245,11 +228,11 @@ class ModulesTest(TestCase):
 
     def tearDown(self):
         """cleanup"""
-        os.chdir(self.cwd)
+        super(ModulesTest, self).tearDown()
+
         os.environ['MODULEPATH'] = os.pathsep.join(self.orig_modulepaths)
         # reinitialize a modules tool, to trigger 'module use' on module paths
         modules_tool()
-        modify_env(os.environ, self.orig_environ)
 
 def suite():
     """ returns all the testcases in this module """
